-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.10.2'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.11.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS VCC QUE ACTUALIZAREMOS
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        -- SELECCIONAMOS LOS VCC EXISTENTES
        SELECT [VCC]
			FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        -- AGREGAMOS LAS RELACIONES ROL-PERMISO CORRESPONDIENTES
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_ADD', N'api/customfield/get_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_ADD', N'api/customfield/get_customfields/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_ADD', N'api/customfield/create_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_ADD', N'api/evaluationformat/get_evaluationformat_attachedsignatures/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_EDIT', N'api/customfield/get_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_EDIT', N'api/customfield/get_customfields/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONFORMAT_EDIT', N'api/customfield/create_customfield/', N'api', GETUTCDATE())


        -- AGREGAMOS LA NUEVA CATEGORIA DE PERMISOS
        INSERT [dbo].[CategoryPermissions] ([VCC], [Category], [Position], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_CUSTOM_FIELD', 20, GETUTCDATE())

        -- AGREGAMOS LOS GRUPOS DE PERMISOS CORRESPONDIENTES
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_CUSTOM_FIELD_ADD', N'CATEG_GROUP_CUSTOM_FIELD', GETUTCDATE(), N'QUALITY')
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_CUSTOM_FIELD_EDIT', N'CATEG_GROUP_CUSTOM_FIELD', GETUTCDATE(), N'QUALITY')
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_CUSTOM_FIELD_DELETE', N'CATEG_GROUP_CUSTOM_FIELD', GETUTCDATE(), N'QUALITY')
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_CUSTOM_FIELD_VIEW', N'CATEG_GROUP_CUSTOM_FIELD', GETUTCDATE(), N'QUALITY')

        -- AGREGAMOS LAS RELACIONES ROL-PERMISO CORRESPONDIENTES
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_CUSTOM_FIELD_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_CUSTOM_FIELD_EDIT', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_CUSTOM_FIELD_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_CUSTOM_FIELD_VIEW', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_CUSTOM_FIELD_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_CUSTOM_FIELD_EDIT', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_CUSTOM_FIELD_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_CUSTOM_FIELD_VIEW', GETUTCDATE())
        
        -- AGREGAMOS LOS PERMISOS PARA EL NUEVO MODULO
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_VIEW', N'speechanalytic/evaluation/custom-field', N'client', GETUTCDATE())        
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_VIEW', N'api/customfield/get_customfields/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_ADD', N'speechanalytic/evaluation/custom-field/add', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_ADD', N'api/customfield/get_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_ADD', N'api/customfield/create_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_EDIT', N'speechanalytic/evaluation/custom-field/edit/', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_EDIT', N'api/customfield/get_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_EDIT', N'api/customfield/save_customfield/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_CUSTOM_FIELD_DELETE', N'api/customfield/delete_customfield', N'api', GETUTCDATE())
        
        
        
        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    -- AGREGAMOS LOS INSERT PARA LAS FIRMAS ADJUNTAS POR DEFECTO DEL SISTEMA
    INSERT INTO [dbo].[EvaluationFormatAttachedSignature] ([VCC], [Description], [Order], [EvaluationFormatId], [CreatedByUserId], [CreatedDate], [TimeStamp]) VALUES (N'system', N'ATTACHED_SIGNATURE_AGENT', 1, NULL, N'system', GETUTCDATE(), GETUTCDATE())
    INSERT INTO [dbo].[EvaluationFormatAttachedSignature] ([VCC], [Description], [Order], [EvaluationFormatId], [CreatedByUserId], [CreatedDate], [TimeStamp]) VALUES (N'system', N'ATTACHED_SIGNATURE_COORDINATOR', 2, NULL, N'system', GETUTCDATE(), GETUTCDATE())
    INSERT INTO [dbo].[EvaluationFormatAttachedSignature] ([VCC], [Description], [Order], [EvaluationFormatId], [CreatedByUserId], [CreatedDate], [TimeStamp]) VALUES (N'system', N'ATTACHED_SIGNATURE_AUDITOR', 3, NULL, N'system', GETUTCDATE(), GETUTCDATE())
    INSERT INTO [dbo].[EvaluationFormatAttachedSignature] ([VCC], [Description], [Order], [EvaluationFormatId], [CreatedByUserId], [CreatedDate], [TimeStamp]) VALUES (N'system', N'ATTACHED_SIGNATURE_RESPONSIBLE', 4, NULL, N'system', GETUTCDATE(), GETUTCDATE())
    
    

    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH

