PRINT N'Iniciando recompilación de la tabla [dbo].[AuditorEvaluationPlan]...';


GO
BEGIN TRANSACTION;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

SET XACT_ABORT ON;

CREATE TABLE [dbo].[tmp_ms_xx_AuditorEvaluationPlan] (
    [Instance]             VARCHAR (50)   NOT NULL,
    [IdEvaluationPlan]     VARCHAR (50)   NOT NULL,
    [IdAuditor]            VARCHAR (50)   NOT NULL,
    [IdInteraction]        VARCHAR (250)  NOT NULL,
    [Completed]            BIT            NULL,
    [Score]                NUMERIC (5, 2) NULL,
    [AutomaticScore]       NUMERIC (5, 2) NULL,
    [ManualScore]          NUMERIC (5, 2) NULL,
    [IdCampaign]           VARCHAR (50)   NOT NULL,
    [IdAgent]              VARCHAR (50)   NOT NULL,
    [InteractionDate]      DATETIME       NULL,
    [InteractionType]      VARCHAR (50)   NOT NULL,
    [InteractionDuration]  NUMERIC (8)    NULL,
    [InteractionOrigin]    VARCHAR (50)   NULL,
    [InteractionFormat]    VARCHAR (50)   NULL,
    [InteractionDateEnd]   DATETIME       NULL,
    [ManagementResult]     VARCHAR (50)   NULL,
    [ContactId]            VARCHAR (50)   NULL,
    [Status]               BIT            CONSTRAINT [DF_AuditorEvaluationPlan_IsDeleted] DEFAULT ((1)) NULL,
    [CreatedByUserId]      VARCHAR (50)   NULL,
    [CreatedDate]          DATETIME       NULL,
    [LastModifiedByUserId] VARCHAR (50)   NULL,
    [LastModifiedDate]     DATETIME       NULL,
    [TimeStamp]            DATETIME       NULL,
    CONSTRAINT [tmp_ms_xx_constraint_PK_AuditorEvaluationPlan_11] PRIMARY KEY CLUSTERED ([Instance] ASC, [IdEvaluationPlan] ASC, [IdAuditor] ASC, [IdInteraction] ASC, [IdAgent] ASC, [InteractionType] ASC)
);

IF EXISTS (SELECT TOP 1 1 
           FROM   [dbo].[AuditorEvaluationPlan])
    BEGIN
        INSERT INTO [dbo].[tmp_ms_xx_AuditorEvaluationPlan] ([Instance], [IdEvaluationPlan], [IdAuditor], [IdInteraction], [IdAgent], [InteractionType], [TimeStamp], [Completed], [Score], [AutomaticScore], [ManualScore], [LastModifiedByUserId], [LastModifiedDate], [IdCampaign], [InteractionDate], [InteractionDuration], [InteractionOrigin], [InteractionFormat], [InteractionDateEnd], [ManagementResult], [ContactId])
        SELECT   [Instance],
                 [IdEvaluationPlan],
                 [IdAuditor],
                 [IdInteraction],
                 [IdAgent],
                 [InteractionType],
                 [TimeStamp],
                 [Completed],
                 [Score],
                 [AutomaticScore],
                 [ManualScore],
                 [LastModifiedByUserId],
                 [LastModifiedDate],
                 [IdCampaign],
                 [InteractionDate],
                 [InteractionDuration],
                 [InteractionOrigin],
                 [InteractionFormat],
                 [InteractionDateEnd],
                 [ManagementResult],
                 [ContactId]
        FROM     [dbo].[AuditorEvaluationPlan]
        ORDER BY [Instance] ASC, [IdEvaluationPlan] ASC, [IdAuditor] ASC, [IdInteraction] ASC, [IdAgent] ASC, [InteractionType] ASC;
    END

DROP TABLE [dbo].[AuditorEvaluationPlan];

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_AuditorEvaluationPlan]', N'AuditorEvaluationPlan';

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_constraint_PK_AuditorEvaluationPlan_11]', N'PK_AuditorEvaluationPlan_1', N'OBJECT';

COMMIT TRANSACTION;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED;


GO
PRINT N'Creando [dbo].[AuditorEvaluationPlan].[IX_AuditorEvaluationPlan]...';


GO
CREATE NONCLUSTERED INDEX [IX_AuditorEvaluationPlan]
    ON [dbo].[AuditorEvaluationPlan]([Instance] ASC, [IdAuditor] ASC, [IdCampaign] ASC, [IdAgent] ASC, [InteractionType] ASC, [InteractionOrigin] ASC, [InteractionFormat] ASC, [ManagementResult] ASC);


GO
PRINT N'Creando [dbo].[AuditorEvaluationPlan].[IX_AuditorEvaluationPlan_IdEvaluationPlan]...';


GO
CREATE NONCLUSTERED INDEX [IX_AuditorEvaluationPlan_IdEvaluationPlan]
    ON [dbo].[AuditorEvaluationPlan]([IdEvaluationPlan] ASC)
    INCLUDE([Instance], [IdAuditor], [IdInteraction], [IdAgent], [InteractionType]);


GO
PRINT N'Modificando [dbo].[GetAuditorEvaluationPlanEvaluation]...';


GO
ALTER PROCEDURE [dbo].[GetAuditorEvaluationPlanEvaluation] (
	@Instance Varchar(50) = NULL,
	@AuditorId Varchar(50) = NULL,
	@EvaluationPlanId Varchar(50) = NULL,
	@InteractionId Varchar(250) = NULL,
	@InteractionType Varchar(25) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT AEP.[IdEvaluationPlan]				AS 'evaluationPlanId', 
			EP.[Evaluation_Mode]				AS 'evaluation_mode',
			AEP.[IdAuditor]						AS 'auditorId', 
			AEP.[IdInteraction]					AS 'interactionId', 
			AEP.[Completed]						AS 'evaluationCompleted', 
			EP.[Description]					AS 'evaluationPlanDescription',
			EP.[IdEvaluationFormat]				AS 'evaluationFormatId',
			AEP.[Completed]						AS 'completed',
			AEP.[Status]						AS 'status'
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = AEP.[Instance]
				AND EP.ID = AEP.IdEvaluationPlan
		WHERE AEP.[Instance] = @Instance 
			AND AEP.[IdEvaluationPlan] = @EvaluationPlanId 
			AND AEP.[IdAuditor] = @AuditorId 
			AND AEP.[IdInteraction] = @InteractionId
			AND AEP.[InteractionType] = @InteractionType;

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetEvaluationPlans]...';


GO
ALTER  PROCEDURE [dbo].[GetEvaluationPlans] (
@Instance           Varchar(50) = NULL,
@SearchCriteria     Varchar(50) = NULL,
@SearchExpression   Varchar(50) = NULL,
@Disabled			bit = NULL
)
AS
BEGIN
SET NOCOUNT ON

    SELECT
			e.[ID]							'id',
			e.[Modality]					'modality',
			e.[DateFin]						'dateFin',
			e.CreatedDate					'createdDate',
			e.CreatedByUserId				'createdByUserId',
			e.LastModifiedDate				'lastModifiedDate',
			e.LastModifiedByUserId			'lastModifiedByUserId',
			e.Description					'description',
			e.IdEvaluationFormat            'idEvaluationFormat',
			e.Enabled						'enabled',
			e.Status						'status',
			e.Evaluation_Mode				'evaluation_mode',
			e.processId    				    'processId',
			ef.templateId    				'templateId',
			count(a.IdInteraction)			'totalEvaluations',
			--ISNULL(sum(cast(a.Completed as int)), 0)   'completedEvaluations',
			--ISNULL(ROUND(sum(cast(a.Completed as numeric(5,2)))*100/count(a.IdInteraction),2), 0) 'completedPercentage',
			COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)					'completedEvaluations',
			(
				(
					CAST(
						COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)
						+ 
						COUNT(DISTINCT CASE a.[Status] WHEN 0 THEN a.[IdInteraction] ELSE NULL END)
					 AS NUMERIC(6,2)) 
				)
				/
				CASE COUNT(DISTINCT a.[IdInteraction]) WHEN 0 THEN 1 ELSE COUNT(DISTINCT a.[IdInteraction]) END 
			) * 100																							'completedPercentage',
			e.[Percentage]					'percentage',
			e.[StatusDescription]			'statusDescription',
			e.[BatchStatus]					'batchStatus'
		FROM EvaluationPlan e (NOLOCK)
		LEFT JOIN AuditorEvaluationPlan a (NOLOCK) 
			ON a.[Instance] = e.[Instance]
				AND a.[IdEvaluationPlan] = e.[ID]
		LEFT JOIN EvaluationFormat ef (NOLOCK)
			ON ef.[Instance] = e.[Instance]
				AND ef.[ID] = e.[IdEvaluationFormat]
		WHERE e.Instance = @Instance
			AND ISNULL(e.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL(e.Modality, '') LIKE CASE WHEN @SearchCriteria = 'modality' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL(@Disabled, 0) <> e.Enabled
		GROUP BY e.[ID],e.[Modality],e.[DateFin],e.CreatedDate,e.CreatedByUserId,
				 e.LastModifiedDate,e.LastModifiedByUserId,e.Description,e.IdEvaluationFormat,
				 e.Enabled, e.Status, e.Evaluation_Mode,e.processId,ef.templateId, 
				 e.Percentage, e.StatusDescription, e.[BatchStatus]	

    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveAuditorEvaluationPlan]...';


GO

ALTER PROCEDURE [dbo].[SaveAuditorEvaluationPlan] (
	@Instance Varchar(50) = NULL, 				
	@IdEvaluationPlan Varchar(50) = NULL,
	@IdAuditor Varchar(50) = NULL,				
	@IdInteraction Varchar(250) = NULL,
	@InteractionType varchar(50) = NULL,	
	@InteractionDate Datetime = NULL,
	@InteractionDateEnd Datetime = NULL,	
	@InteractionDuration numeric(8) = NULL,
	@IdAgent varchar(50) = NULL,
	@idCampaign varchar(50) = NULL,
	@InteractionDispositionCode varchar(50) = NULL,
	@InteractionOrigin varchar(50) = NULL,
	@InteractionFormat varchar(20) = NULL,
	@ContactId varchar(50) = NULL,
	@LastModifiedByUserId Varchar(100) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @ID INT = 0;

	-- Verificamos si existen un registros con las datos recibidos
	SELECT @ID = COUNT([Instance])
		FROM [AuditorEvaluationPlan] AEP (NOLOCK)
		WHERE AEP.[Instance] = @Instance
			AND AEP.[IdEvaluationPlan] = @IdEvaluationPlan
			AND AEP.[IdAuditor] = @IdAuditor
			AND AEP.[IdInteraction] = @IdInteraction
			AND AEP.[IdAgent] = @IdAgent
			AND AEP.[InteractionType] = @InteractionType

	-- Si existe, actualizamos dicho registro
	IF @ID > 0
		BEGIN
			UPDATE [AuditorEvaluationPlan]
				SET [InteractionOrigin] = @InteractionOrigin,
					[InteractionFormat] = @InteractionFormat,
					[ContactId] = [ContactId],
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [Instance] = @Instance
					AND [IdEvaluationPlan] = @IdEvaluationPlan
					AND [IdAuditor] = @IdAuditor
					AND [IdInteraction] = @IdInteraction
					AND [IdAgent] = @IdAgent
					AND [InteractionType] = @InteractionType
		END
	-- Si no existe lo insertamos
	ELSE 
		BEGIN
			INSERT INTO [AuditorEvaluationPlan] (
				[Instance], [IdEvaluationPlan], [IdAuditor],
				[IdInteraction], [InteractionType], [InteractionDate],
				[InteractionDateEnd], [ManagementResult], [InteractionOrigin], 
				[IdAgent], [idCampaign], [InteractionDuration],
				[InteractionFormat], [ContactId], [TimeStamp], 
				[Completed], [Score], [LastModifiedByUserId],
				[Status]
				)
			VALUES (
				@Instance, @IdEvaluationPlan, @IdAuditor,
				@IdInteraction, @InteractionType, @InteractionDate, 
				@InteractionDateEnd, @InteractionDispositionCode, @InteractionOrigin,
				@IdAgent, @IdCampaign, @InteractionDuration,
				@InteractionFormat, @ContactId, GETUTCDATE(), 
				0, 0, @LastModifiedByUserId,
				1
			)
		END
	
    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveEvaluationPlanCompleted]...';


GO

ALTER PROCEDURE [dbo].[SaveEvaluationPlanCompleted] (
	@Instance Varchar(50) = NULL, 
	@IdEvaluationPlan Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[EvaluationPlan]
		SET [Status] = 7,
			[Percentage] = 1,
			[StatusDescription] = CASE WHEN [ERROR] = 0 THEN 'COMPLETED' ELSE 'COMPLETED_WITH_ERROR' END
		WHERE NOT EXISTS (
			SELECT AEP.[idEvaluationPLan]
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
					AND AEP.[Instance] = @Instance
					AND AEP.[Completed] <> 1
					AND AEP.[Status] = 1
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan

	-- ACTUALIZAMOS EL STATUS DESCRIPTION CUANDO EXISTEN DESCARTADOS
	/*
	UPDATE [dbo].[EvaluationPlan]
		SET [StatusDescription] = 'COMPLETED_WITH_DISCARDED'
		WHERE EXISTS (
				SELECT AEP.[idEvaluationPLan]
					FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
						AND AEP.[Instance] = @Instance
						AND AEP.[Status] = 0
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND (
				[Status] = 7
				OR
				[Status] = 8
			)
	*/

	-- ACTUALIZAMOS EL STATUS DESCRIPTION CUANDO EXISTEN DESCARTADOS
	UPDATE EP 
		SET EP.[StatusDescription] = 'COMPLETED_WITH_DISCARDED',
			--EP.[Total] = INNER_AEP.[Total],
			--EP.[Progress] = INNER_AEP.[Completed] + INNER_AEP.[Discarded]
			EP.[Percentage] = (INNER_AEP.[Completed] + INNER_AEP.[Discarded]) / INNER_AEP.[Total]
		FROM [dbo].[EvaluationPlan] EP 
		INNER JOIN (
			SELECT AEP.[Instance],
					AEP.[idEvaluationPlan],
					COUNT(AEP.[IdInteraction])														'Total',
					COUNT(CASE AEP.[Completed] WHEN 1 THEN 1 ELSE NULL END)							'Completed',
					COUNT(CASE AEP.[Status] WHEN 0 THEN 1 ELSE NULL END)							'Discarded',
					COUNT(CASE 
							WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN 1 
							ELSE NULL 
							END
					)																				'Pending'
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[Instance] = @Instance
					AND AEP.[idEvaluationPlan] = @IdEvaluationPlan
				GROUP BY AEP.[Instance], AEP.[idEvaluationPlan]
		) INNER_AEP
			ON INNER_AEP.[Instance] = EP.[Instance]
				AND INNER_AEP.[IdEvaluationPlan] = EP.[ID] 
		WHERE EP.[Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND (
				(
					EP.[Status] = 7
					AND 
					INNER_AEP.[Discarded] > 0
				)
				OR
				(
					EP.[Status] = 8
					AND
					INNER_AEP.[Pending] = 0
					AND 
					INNER_AEP.[Total] = INNER_AEP.[Discarded]
				)
			)

	UPDATE [dbo].[EvaluationPlan]
		SET [Status] = 6
		WHERE EXISTS (
			SELECT AEP.[idEvaluationPLan]
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
					AND AEP.[Instance] = @Instance
					AND AEP.[Completed] = 1
					AND AEP.[Status] = 1
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND [status] = 1
			AND [Evaluation_Mode] = 'M'

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetEvaluationPlansAuditor]...';


GO

ALTER PROCEDURE [dbo].[GetEvaluationPlansAuditor] (
	@Instance VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,
	@EndDate VARCHAR(50) = NULL,
	@EvaluationPlans VARCHAR(MAX) = NULL,
	@EvaluationFormats VARCHAR(MAX) = NULL,
	@Modalities VARCHAR(MAX) = NULL,
	@Completed BIT = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = ''

	-- IDENTIFICAMOS EL ROL DEL USUARIO
	SELECT @USER_ROLE = U.[Role]
		FROM [dbo].[User] U (NOLOCK)
		WHERE U.[VCC] = @Instance
			AND U.[ID] = @UserId

	-- ARMAMOS EL SELECT DE LA QURY
	SET @SQL_SELECT_1 += '
		SELECT EP.[ID]																								''id'', 
				EP.[Description]																					''description'',
				EP.[Modality]																						''modality'',
				EP.[DateFin]																						''dateFin'',      
				EP.[CreatedDate]																					''createdDate'',      
				EP.[IdEvaluationFormat]																				''idEvaluationFormat'',
				COUNT(DISTINCT AEP.[IdInteraction])																	''totalInteraction'',
				COUNT(DISTINCT CASE AEP.[Completed] WHEN 1 THEN AEP.[IdInteraction] ELSE NULL END)					''completedInteraction'',      
				COUNT(DISTINCT CASE AEP.[Status] WHEN 0 THEN AEP.[IdInteraction] ELSE NULL END)						''discardedInteraction'',
				-- COUNT(DISTINCT CASE AEP.[Completed] WHEN 0 THEN AEP.[IdInteraction] ELSE NULL END)				''pendingInteraction'',
				COUNT(DISTINCT
					CASE 
					WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN AEP.[IdInteraction] 
					ELSE NULL 
					END)																							''pendingInteraction'',
				SUM(DISTINCT CASE AEP.[Completed] WHEN 0 THEN AEP.[InteractionDuration] ELSE 0 END)					''pendingInteractionTime'',
				COUNT(AEP.[IdInteraction])																			''totalEvaluation'',    
				COUNT(CASE AEP.[Completed] WHEN 1 THEN 1 ELSE NULL END)												''completedEvaluation'',      
				COUNT(CASE AEP.[Status] WHEN 0 THEN 1 ELSE NULL END)												''discardedEvaluation'',    
				-- COUNT(CASE AEP.[Completed] WHEN 0 THEN 1 ELSE NULL END)											''pendingEvaluation'',
				COUNT(
					CASE 
					WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN 1 
					ELSE NULL 
					END)																							''pendingEvaluation'',
				SUM(CASE AEP.[Completed] WHEN 0 THEN AEP.[InteractionDuration] ELSE 0 END)							''pendingEvaluationTime'' '

	SET @SQL_FROM += '
			FROM [EvaluationPlan] EP (NOLOCK)
			INNER JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON (AEP.[Instance] = EP.[Instance] 
					AND AEP.[IdEvaluationPlan] = EP.[ID]'

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		SET @SQL_FROM += '
					AND AEP.[IdAuditor] = ''' + @UserId + ''') '
	END
	ELSE IF (@USER_ROLE = 'administrator' OR @USER_ROLE = 'quality_admin')
	BEGIN
		SET @SQL_FROM += ')'
	END
	ELSE 
	BEGIN
		SET @SQL_FROM += '
					AND AEP.[IdAuditor] = ''' + @UserId + ''') '
	END 

	SET @SQL_FILTER += '
			WHERE EP.[Instance] = ''' + @Instance + '''
				AND EP.[DateFin] <= ''' + @EndDate + ''' 
				AND EP.[Enabled] = 1 
				AND (
					(
						EP.[Evaluation_Mode] = ''X'' 
						AND 
						(
							EP.[Status] = 7
							OR
							EP.[Status] = 8
						)
					)
					OR
					(EP.[Evaluation_Mode] = ''M'')
				) '

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		SET @SQL_FILTER += '
				AND EP.[IdEvaluationFormat] IN (
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF 
						WHERE CEF.[Instance] = ''' + @Instance + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END

	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans <> '') SET @SQL_FILTER += 'AND EP.[ID] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	IF (@EvaluationFormats IS NOT NULL AND @EvaluationFormats <> '') SET @SQL_FILTER += 'AND EP.[IdEvaluationFormat] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationFormats + ''','','')) '
	IF (@Modalities IS NOT NULL AND @Modalities <> '') SET @SQL_FILTER += 'AND EP.[Modality] IN (SELECT Value FROM [dbo].[split](''' + @Modalities + ''','','')) '
	
	-- VARIABLE PARA FILTRAR SOLO LOS REGISTROS DE AUDITOREVALUATIONPLAN CON [COMPLETED] = 1
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 '
		SET @SQL_FILTER_2 = ' '
	END
	ELSE 
	BEGIN
		-- SI NO LLEGA UN VALOR PARA EVALUATION PLANS, BUSCAMOS SOLO LOS QUE TIENEN INTERACCIONES PENDIENTES
		IF (@EvaluationPlans IS NULL OR @EvaluationPlans = '')
		BEGIN
			--SET @SQL_FILTER += 'AND AEP.[Completed] = 0 '
			SET @SQL_FILTER_2 = 'WHERE X.[pendingInteraction] > 0 '
		END
	END
	

	SET @SQL_EXTRA += '
			GROUP BY EP.[ID],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]'

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA

	--SET @SQL_SELECT_1 = 'SELECT * FROM (' + @SQL_SELECT_1 + ') AS X WHERE X.[pendingInteractionCount] > 0 ORDER BY X.[Description] ASC; '
	SET @SQL_SELECT_2 = 'SELECT * '
	SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1 + ') AS X '
	-- @SQL_FILTER_2 YA FUE DEFINIDO SEGUN SE BUSCAN LAS YA EVALUADAS O NO EVALUADAS
	SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC'

	SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2

	-- select @SQL_SELECT_2;
	EXEC (@SQL_SELECT_2)
	
	SET NOCOUNT OFF
	
END
GO
PRINT N'Modificando [dbo].[GetNotEvaluatedInteractionsAuditor]...';


GO
-- ESTE SP MUETRA LA LISTA DE EVALUACIONES PENDIENTES PARA UN AUDITOR
ALTER PROCEDURE [dbo].[GetNotEvaluatedInteractionsAuditor] (
	@Instance Varchar(50) = NULL, 
	@UserId Varchar(50) = NULL,
	@IdCampaigns VARCHAR(MAX) = NULL,
	@IdAgents Varchar(MAX) = NULL,
	@EvaluationPlans Varchar(MAX) = NULL,
	@InteractionTypes Varchar(100) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
 	@EvaluationDateTo   VARCHAR(MAX) = '2050-01-01T23:59:59',
	@EvaluationStatus BIT = 1,
 	@InteractionIds     VARCHAR(MAX) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''

	-- IDENTIFICAMOS EL ROL DEL USUARIO
	SELECT @USER_ROLE = U.[Role]
		FROM [dbo].[User] U (NOLOCK)
		WHERE U.[VCC] = @Instance
			AND U.[ID] = @UserId

	-- EMULAMOS UNA NUEVA TABLA EN BASE A AuditorEvaluationPlan DONDE SE REMOVERAN TODOS LOS REGISTROS DUPLICADOS PARA EL PK DE LA TABLA Audio
	SET @SQL_SELECT_1 += '
		WITH CTE AS (
			SELECT AEP.[Instance], 
					AEP.[IdEvaluationPlan], 
					AEP.[IdInteraction], 
					AEP.[IdAuditor], 
					AEP.[Score], 
					IEP.[AgentName],
					AEP.[LastModifiedDate],
					RN = ROW_NUMBER() OVER(PARTITION BY AEP.[Instance], AEP.[IdEvaluationPlan], AEP.[IdInteraction] ORDER BY AEP.[LastModifiedDate] DESC)
			FROM AuditorEvaluationPlan AEP (NOLOCK)
			LEFT JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance] 
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
					AND IEP.[IdAgent] = AEP.[IdAgent]
			WHERE AEP.[Instance] = ''' + @Instance + ''' 
				AND AEP.[Completed] = 0 
				AND AEP.[IdAuditor] <> ''SPEECH''
		) '
	-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
	
	-- ARMAMOS EL SELECT PRINCIPAL PARA LA BUSQUEDA DE EVALUACIONES PENDIENTES DEL AUDITOR
	SET @SQL_SELECT_1 += '
		SELECT DISTINCT '
	SET @SQL_SELECT_1 += '
				AEP.[IdInteraction]															''id'', 
				AEP.[IdCampaign]															''descriptionCampaign'',
				AEP.[IdAgent]																''idAgent'',
				AEP.[IdAgent]																''descriptionAgent'',
				AEP.[InteractionDate]														''dateIni'',
				AEP.[InteractionDateEnd]													''dateFin'',
				AEP.[InteractionDuration]													''interactionTime'',
				AEP.[Completed]																''evaluated'',
				AEP.[InteractionType]														''interactionType'',
				AEP.[ManagementResult]														''dispositionCode'',
				AEP.[ContactId]																''address'',
				AEP.[InteractionOrigin]														''origin'',
				AEP.[IdEvaluationPlan]														''idEvaluationPlan'',
				EP.[ProcessId]																''processId'',
				EP.[Description]															''descriptionEvaluationPlan'',
				EP.[DateFin]																''dateFinEvaluationPlan'', 
				EP.[Modality]																''evaluationPlanModality'',
				EP.[Evaluation_Mode]														''evaluationPlanEvaluationMode'',
				A.[ID]																		''idAuditor'', 
				A.[Description]																''descriptionAuditor'',
				ISNULL(U.[FirstName] + '' '', '''') + ISNULL(U.[LastName], '''')			''auditorName'',						-- NOMBRE DEL AUDITOR
				C.[AgentName]																''agentName'',							-- NOMBRE DEL AGENTE
				C.[IdAuditor]																''lastEvaluationEvaluator'', 
				C.[Score]																	''lastEvaluationScore'',
				AEP.[Status]																''status'' '

	SET @SQL_FROM += '
			FROM [AuditorEvaluationPlan] AEP (NOLOCK) 
			LEFT JOIN [EvaluationPlan] EP (NOLOCK) 
				ON (EP.[Instance] = EP.Instance
					AND EP.[ID] = AEP.IdEvaluationPlan) 
			LEFT JOIN [dbo].[User] U (NOLOCK)
				ON U.[VCC] = AEP.[Instance]
					AND U.[ID] = AEP.[IdAuditor]
			LEFT JOIN [Auditor] A (NOLOCK) 
				ON (A.[ID] = AEP.[IdAuditor])
			LEFT JOIN CTE C (NOLOCK) 
				ON C.[Instance] = AEP.[Instance] 
					AND C.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
					AND C.[IdInteraction] = AEP.[IdInteraction]
					AND C.[RN] = 1 '

	SET @FILTERS_FIXED_PART = '
			WHERE AEP.[Instance] = ''' + @Instance + ''' 
				AND AEP.[Completed] = 0
				AND EP.[Enabled] = 1 
				AND (
					(EP.[Evaluation_Mode] = ''X'' AND EP.[Status] = 8)
					OR
					(EP.[Evaluation_Mode] = ''M'')
				) '

	-- AGREGAMOS EL FILTRO DE ESTADO
	IF @EvaluationStatus = 0
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 0 '
	END
	ELSE IF @EvaluationStatus = 1
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 1 '
	END

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND	AEP.[IdAuditor] = ''' + @UserId + ''' '
	END
	ELSE IF @USER_ROLE = 'administrator' OR @USER_ROLE = 'quality_admin'
	BEGIN
		SET @FILTERS_FIXED_PART += ''
	END
	ELSE 
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND	AEP.[IdAuditor] = ''' + @UserId + ''' '
	END

	IF (@IdCampaigns IS NOT NULL AND @IdCampaigns != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaigns + ''','','')) ';
	IF (@IdAgents IS NOT NULL AND @IdAgents != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgents + ''','','')) ';
	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) ';
	IF (@InteractionTypes IS NOT NULL AND @InteractionTypes != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionType] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTypes + ''','','')) ';
	-- IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @FILTERS_EXCLUDING_PART += 'AND CONCAT(AEP.[IdCampaign], ''_'', AEP.[ManagementResult]) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND (
								(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
							)'
	IF (@EvaluationDateTo IS NOT NULL AND @EvaluationDateTo != '') SET @FILTERS_OPTIONAL_PART += 'AND EP.[DateFin] <= ''' + @EvaluationDateTo + '''  ';
	
	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
       BEGIN 
            SET @FILTERS_OPTIONAL_PART = '';
	        SET @FILTERS_EXCLUDING_PART=' AND AEP.[IdInteraction] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
       END

	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART;

	--SELECT @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)

 	SET NOCOUNT OFF
END
GO
PRINT N'Creando [dbo].[SaveEvaluationStatus]...';


GO
CREATE PROCEDURE [dbo].[SaveEvaluationStatus]
(
	@VCC							VARCHAR(50) = NULL,
	@EvaluationPlanId				VARCHAR(50) = NULL,
	@InteractionId					VARCHAR(250) = NULL,
	@AuditorId						VARCHAR(50) = NULL,
	@Status							BIT = NULL,
	@Multiple						BIT = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	--
	UPDATE [dbo].[AuditorEvaluationPlan]	
		SET [Status] = @Status,
			[LastModifiedByUserId] = @LastModifiedByUserId,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [Instance] = @VCC
			AND [IdEvaluationPlan] = @EvaluationPlanId
			AND [IdInteraction] = @InteractionId
			AND [IdAuditor] = CASE @Multiple WHEN 0 THEN @AuditorId ELSE [IdAuditor] END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
PRINT N'Actualizando [dbo].[BulkAbilityCognitive]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkAbilityCognitive]';


GO
PRINT N'Actualizando [dbo].[BulkEvaluationPlanAbility]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkEvaluationPlanAbility]';


GO
PRINT N'Actualizando [dbo].[BulkEvaluationPlanQuery]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkEvaluationPlanQuery]';


GO
PRINT N'Actualizando [dbo].[BulkProgress]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkProgress]';


GO
PRINT N'Actualizando [dbo].[DeleteAuditorEvaluationPlan]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[DeleteAuditorEvaluationPlan]';


GO
PRINT N'Actualizando [dbo].[DeleteAuditorEvaluationPlanByInteraction]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[DeleteAuditorEvaluationPlanByInteraction]';


GO
PRINT N'Actualizando [dbo].[DeleteEvaluationPlan]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[DeleteEvaluationPlan]';


GO
PRINT N'Actualizando [dbo].[GetAuditorsEvaluationPlan]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAuditorsEvaluationPlan]';


GO
PRINT N'Actualizando [dbo].[GetAuditorsInteractions]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAuditorsInteractions]';


GO
PRINT N'Actualizando [dbo].[GetCampaignFulfillment]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetCampaignFulfillment]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationPlansByStatus]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationPlansByStatus]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationResultsFromAuditorEvaluationPlan]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationResultsFromAuditorEvaluationPlan]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationsOfInteraction]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationsOfInteraction]';


GO
PRINT N'Actualizando [dbo].[GetInteractionsByType]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionsByType]';


GO
PRINT N'Actualizando [dbo].[GetInteractionsEvaluationPlanWithoutFilters]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionsEvaluationPlanWithoutFilters]';


GO
PRINT N'Actualizando [dbo].[GetTopAgents]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetTopAgents]';


GO
PRINT N'Actualizando [dbo].[ReportByDetailAgent]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[ReportByDetailAgent]';


GO
PRINT N'Actualizando [dbo].[SaveAuditorEvPlanResult]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveAuditorEvPlanResult]';


GO
PRINT N'Actualizando [dbo].[SaveEvaluationFormat]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveEvaluationFormat]';


GO
PRINT N'Actualizando [dbo].[SaveInteractionEvaluationPlanResult]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveInteractionEvaluationPlanResult]';


GO
PRINT N'Actualización completada.';


GO
