-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.16.0'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.16.1'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION





    -- ACTUALIZAMOS DISTRIBUTE DE LAS HABILIDADES YA EXISTENTES
    UPDATE [dbo].[Ability] SET [Distribute] = 0    

    -- ACTUALIZAMOS EN LOS NUEVOS CAMPOS SCORES PROPIOS Y HEREDADOS DE LOS EVALUATIONRESULTS EXISTENTES
    UPDATE [dbo].[EvaluationResult] SET [OwnScore] = [Qualification], [InheritedScore] = 0

    -- ACTUALIZAMOS LOS EVALUATION RESULT INCLUYENDO EL PONDERADO INTERNO DE LA HABILIDAD
    UPDATE ER 
        SET ER.[AbilityInnerScore] = S.[Value]
        FROM [dbo].[EvaluationResult] ER 
        INNER JOIN [dbo].[Score] S 
            ON S.[Instance] = ER.[Instance]
                AND S.[IdAbility] = ER.[IdAbility]
                AND S.[Description] = ER.[ScoreDescription]

    -- ACTUALIZAMOS EL CAMPO EXISTENTE EN EVALUATION RESULT PARA EL PROBLEMA DE PUNTAJE EN EVALUACIONES AUTOMATICAS
    UPDATE ER 
        SET ER.[MaxAbilityScore] = AEF.[Score]
        FROM [dbo].[EvaluationResult] ER
        INNER JOIN [dbo].[AbilityEvaluationFormat] AEF 
            ON AEF.[Instance] = ER.[Instance]
                AND AEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
                AND AEF.[ID] = ER.[IdAbilityEvaluationFormat]
        WHERE ER.[IdAuditor] = 'SPEECH' 
            OR ER.[IdEvaluator] = 'SPEECH'
    
    

    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH

