-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.18.0'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.19.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    


    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
    FOR 
    -- SELECCIONAMOS LOS VCC EXISTENTES
    SELECT [VCC]
        FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE`
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_VIEW', N'api/interaciontype/get_interactiontypes', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_NLU_VIEW', N'api/nlu/nlu-to-export/', N'api', GETUTCDATE())
   
        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR




    -- ACTUALIZAMOS EL CAMPO COGNITIVE ENGINE DE LOS NLU EXISTENTES
    UPDATE [dbo].[NLU] SET [CognitiveEngine] = 'RASA' WHERE [IsSystemApp] = 1
    UPDATE [dbo].[NLU] SET [CognitiveEngine] = 'RASA' WHERE [IsSystemApp] = 0 AND [CognitiveEngine] IS NULL AND [AppId] LIKE '%::%'
    UPDATE [dbo].[NLU] SET [CognitiveEngine] = 'LUIS' WHERE [IsSystemApp] = 0 AND [CognitiveEngine] IS NULL AND [AppId] NOT LIKE '%::%'
    -- ACTUALIZAMOS EL CAMPO DE LA TABLA NLU PARA INDICAR AL TASK QUE DEBE IMPORTAR LA DATA DE LOS NLU A BD
    UPDATE [dbo].[NLU] SET [LastModifiedByUserId] = 'dev' WHERE ([IsSystemApp] = 1 AND [LastTrainedDate] IS NOT NULL) OR ([IsSystemApp] = 0)
    -- ACTUALIZAMOS LOS CAMPOS DE LA TABLA NLU PARA SETTEAR AVAILABLE POR DEFECTO A TODOS LOS MODELOS
    UPDATE [dbo].[NLU] SET [Status] = 1, [StatusCode] = 'AVAILABLE'
    


    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
    FOR 
    -- SELECCIONAMOS LOS VCC EXISTENTES
    SELECT [VCC],
            [AppId]
        FROM [dbo].[NLU] (NOLOCK) 
        WHERE [IsSystemApp] = 1

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC VARCHAR(50) = ''
    DECLARE @CURRENT_NLU VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC, @CURRENT_NLU
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- INSERTAMOS EL INTENT AGENT A CADA MODELO DE DIARIZACION
        INSERT INTO [dbo].[Intent] ([VCC], [Name], [IntentId], [AppId], [CreatedDate], [CreatedByUserId], [TimeStamp])
            VALUES (@CURRENT_VCC, 'Agent', 'Agent', @CURRENT_NLU, GETUTCDATE(), 'system', GETUTCDATE())
            
        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC, @CURRENT_NLU
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    
    

    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, 
            [IsRegistered] = 0,
            [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH



