-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.20.1'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.21.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION





    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION
    --ACTUALIZAR TemplateTheme CON UN VALOR POR DEFECTO
    UPDATE [dbo].[TemplateTheme] SET [IsNegative] = 0
    -- ACTUALIZAR POR DEFECTO LOS DETALLES DE ESTUDIOS
    UPDATE [dbo].[StudyDetail] SET [IsMatched] = 1
    UPDATE [dbo].[StudyResultSummary] SET [IsMatched] = 1
    UPDATE [dbo].[StudyResultSummaryDetail] SET [IsMatched] = 1
    --Actualizar TemplateThemeItem CON UN VALOR POR DEFECTO
    UPDATE [dbo].[TemplateThemeItem] SET [AcceptancePercentage]  = CAST(50 AS DECIMAL(5,2)) 
    
    --ACTUALIZAR AnalysisType PARA TIPO QUERY   
    UPDATE TT SET TT.[AnalysisType] = 'SEMANTIC'
        FROM TemplateTheme TT 
        JOIN TemplateThemeItem TTI 
            ON TT.VCC = TTI.VCC
                AND TT.TemplateId = TTI.TemplateId
                AND TT.ID = TTI.TemplateThemeId
        JOIN GenericQuery GQ 
            ON GQ.VCC = TT.VCC 
                AND GQ.Name = TTI.ID

    --ACTUALIZAR AnalysisType CON  TIPO NLU 
    UPDATE TemplateTheme SET AnalysisType ='COGNITIVE' WHERE AnalysisType IS NULL 

    -- ACTUALIZAMOS LOS REGISTROS DE NLU PARA ENTRENAR LOS MODELOS RASA DESPUES DE ACTUALIZAR RASA
    UPDATE [dbo].[NLU] 
        SET [NeedTrain] = 1, 
            [LastModifiedByUserId] = 'dev-2', 
            [LastModifiedDate] = GETUTCDATE() 
        WHERE [CognitiveEngine] = 'RASA' 
            AND [NeedTrain] = 0 
            AND [LastTrainedDate] IS NOT NULL

    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
    FOR 
    -- SELECCIONAMOS LOS VCC EXISTENTES
    SELECT [VCC]
        FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE`
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (N'system', NEWID(), N'PERM_GROUP_TEMPLATE_ADD', N'api/entity/get_nlus_entities', N'api', GETUTCDATE())
     
        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR
    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION



    

    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, 
            [IsRegistered] = 0,
            [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH



