-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.4.2'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.5.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    
        -- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS TIPOS DE UPDATE
        DECLARE RECURRENT_CURSOR CURSOR 
        LOCAL STATIC READ_ONLY FORWARD_ONLY
            FOR 
            SELECT DISTINCT [VCC]
                -- FROM TMP_TABLE_UPDATE ASAVDTTMP 
                FROM [dbo].[VCC] 

        -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
        DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

        -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
        OPEN RECURRENT_CURSOR
        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
        WHILE @@FETCH_STATUS = 0
        BEGIN 
            
            -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

            -- SE DEBERA INSERTAR POR CADA VCC FUNCIONANDO
            INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_VCC_ADD', N'api/vcc/get_integrated_vccs', N'api', CAST(N'2020-06-16T00:00:00.000' AS DateTime))
            INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_VCC_EDIT', N'api/vcc/get_integrated_vccs', N'api', CAST(N'2020-06-16T00:00:00.000' AS DateTime))
            -- AGREGADOS 20200716 (AGREGAR AL CURSOR DE ARRIBA)
            INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_VIEW', N'api/interaction/get_interaction_view', N'api', CAST(N'2020-07-16T00:00:00.000' AS DateTime))
            -- AGREGADOS 20200721 (AGREGAR AL CURSOR DE ARRIBA)
            INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATION_VIEW', N'api/interaction/get_interaction_view', N'api', CAST(N'2020-07-16T00:00:00.000' AS DateTime))
            
            -- IMPORTANTE: CADA VEZ QUE SE AGREGUE PERMISOS SE DEBE LIMPIAR LA KEY DE REDIS ROLE

            -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

            FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
        END
        CLOSE RECURRENT_CURSOR
        DEALLOCATE RECURRENT_CURSOR

        -- ACTUALIZAMOS A INTERACTIONTYPE CALL TODOS LOS REGISTROS EN LA TABLA TRANSCRIPTION
        UPDATE [dbo].[Transcription] SET [InteractionType] = 'CALL' WHERE [InteractionType] IS NULL

        -- ACTUALIZAMOS EL VALOR EN EL NUEVO CAMPO PARA LAS INTERACCIONES CALL
        UPDATE [dbo].[Audio] SET [Channel] = 'CALL' WHERE [Channel] IS NULL
        

    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH