PRINT N'Dropping [dbo].[DF_Audio_Char]...';


GO
ALTER TABLE [dbo].[Audio] DROP CONSTRAINT [DF_Audio_Char];


GO
PRINT N'Dropping [dbo].[DF_Audio_Words]...';


GO
ALTER TABLE [dbo].[Audio] DROP CONSTRAINT [DF_Audio_Words];


GO
PRINT N'Dropping [dbo].[DF_Audio_TotalPhrases]...';


GO
ALTER TABLE [dbo].[Audio] DROP CONSTRAINT [DF_Audio_TotalPhrases];


GO
PRINT N'Dropping [dbo].[DF_Audio_CharsPerPhrase]...';


GO
ALTER TABLE [dbo].[Audio] DROP CONSTRAINT [DF_Audio_CharsPerPhrase];


GO
PRINT N'Starting rebuilding table [dbo].[Audio]...';


GO
BEGIN TRANSACTION;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

SET XACT_ABORT ON;

CREATE TABLE [dbo].[tmp_ms_xx_Audio] (
    [VCC]                    VARCHAR (50)    NOT NULL,
    [ID]                     VARCHAR (100)   NOT NULL,
    [Section]                NUMERIC (5)     NOT NULL,
    [Part]                   NUMERIC (5)     NOT NULL,
    [Channel]                VARCHAR (50)    CONSTRAINT [DF_Audio_Channel] DEFAULT ('CALL') NULL,
    [Number]                 VARCHAR (50)    NULL,
    [Gender]                 VARCHAR (50)    NULL,
    [SilenceTime]            NUMERIC (12, 3) NULL,
    [DurationTime]           NUMERIC (12, 3) NULL,
    [DurationLinguisticTime] NUMERIC (12, 3) NULL,
    [ValidChar]              INT             CONSTRAINT [DF_Audio_Char] DEFAULT ((0)) NULL,
    [ValidWords]             INT             CONSTRAINT [DF_Audio_Words] DEFAULT ((0)) NULL,
    [TotalPhrases]           INT             CONSTRAINT [DF_Audio_TotalPhrases] DEFAULT ((0)) NULL,
    [CharsPerPhrase]         INT             CONSTRAINT [DF_Audio_CharsPerPhrase] DEFAULT ((0)) NULL,
    [Category1]              VARCHAR (100)   NULL,
    [Category2]              VARCHAR (100)   NULL,
    [Category3]              VARCHAR (100)   NULL,
    [Description]            VARCHAR (100)   NULL,
    [Campaign]               VARCHAR (100)   NULL,
    [Agent]                  VARCHAR (50)    NULL,
    [Type]                   VARCHAR (50)    NULL,
    [Init]                   VARCHAR (50)    NULL,
    [IsProcess]              BIT             NULL,
    [IsCancel]               BIT             NULL,
    [IsTranscribed]          INT             NULL,
    [StartDate]              DATETIME        NULL,
    [EndDate]                DATETIME        NULL,
    [DispositionCode]        VARCHAR (50)    NULL,
    [FileName]               VARCHAR (200)   NULL,
    [ErrorMessage]           VARCHAR (MAX)   NULL,
    [CreatedDate]            DATETIME        NULL,
    [CreatedByUserId]        VARCHAR (50)    NULL,
    [LastModifiedDate]       DATETIME        NULL,
    [LastModifiedByUserId]   VARCHAR (50)    NULL,
    [TimeStamp]              DATETIME        NULL,
    CONSTRAINT [tmp_ms_xx_constraint_PK_Audio_11] PRIMARY KEY CLUSTERED ([VCC] ASC, [ID] ASC, [Section] ASC, [Part] ASC)
);

IF EXISTS (SELECT TOP 1 1 
           FROM   [dbo].[Audio])
    BEGIN
        INSERT INTO [dbo].[tmp_ms_xx_Audio] ([VCC], [ID], [Section], [Part], [Number], [Gender], [SilenceTime], [DurationTime], [DurationLinguisticTime], [ValidChar], [ValidWords], [TotalPhrases], [CharsPerPhrase], [Category1], [Category2], [Category3], [Description], [Campaign], [Agent], [Type], [Init], [IsProcess], [IsCancel], [IsTranscribed], [StartDate], [EndDate], [DispositionCode], [FileName], [ErrorMessage], [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], [TimeStamp])
        SELECT   [VCC],
                 [ID],
                 [Section],
                 [Part],
                 [Number],
                 [Gender],
                 [SilenceTime],
                 [DurationTime],
                 [DurationLinguisticTime],
                 [ValidChar],
                 [ValidWords],
                 [TotalPhrases],
                 [CharsPerPhrase],
                 [Category1],
                 [Category2],
                 [Category3],
                 [Description],
                 [Campaign],
                 [Agent],
                 [Type],
                 [Init],
                 [IsProcess],
                 [IsCancel],
                 [IsTranscribed],
                 [StartDate],
                 [EndDate],
                 [DispositionCode],
                 [FileName],
                 [ErrorMessage],
                 [CreatedDate],
                 [CreatedByUserId],
                 [LastModifiedDate],
                 [LastModifiedByUserId],
                 [TimeStamp]
        FROM     [dbo].[Audio]
        ORDER BY [VCC] ASC, [ID] ASC, [Section] ASC, [Part] ASC;
    END

DROP TABLE [dbo].[Audio];

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_Audio]', N'Audio';

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_constraint_PK_Audio_11]', N'PK_Audio_1', N'OBJECT';

COMMIT TRANSACTION;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED;


GO
PRINT N'Altering [dbo].[GetStudyResultSummary]...';


GO
ALTER PROCEDURE [dbo].[GetStudyResultSummary]
(
	@VCC					VARCHAR(50),
	@StudyId				VARCHAR(200),
	@NewStructure			VARCHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		SELECT
			B.[Name]						'processName',
			A.[Name]						'studyName',
			A.[Description]					'studyDescription',
			A.[TemplateId]					'templateId'
		FROM [dbo].[Study] A (NOLOCK)
		INNER JOIN Process B ON A.ProcessId = B.[Name]
		WHERE A.[VCC] = @VCC 
			AND A.[Name] = @StudyId
	END

	BEGIN
		SELECT
			[VCC]							'vcc',
			[StudyId]						'studyId',
			[TemplateId]					'templateId',
			[TemplateThemeId]				'templateThemeId',
			[ID]							'id',
			[Description]					'description',
			[Category1]						'category1',
			[Category2]						'category2',
			[Total]							'total',
			[Duration]						'duration',
			[Silence]						'silence',
			[Confidence]					'confidence',
			[CreatedDate]					'createdDate',
			[CreatedByUserId]				'createdByUserId',
			[LastModifiedDate]				'lastModifiedDate',
			[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[StudyResultSummary] (NOLOCK)
		WHERE [VCC] = @VCC 
			AND [StudyId] = @StudyId
	END

	IF(@NewStructure IS NULL)
	BEGIN
		SELECT
			A.[TemplateThemeId]									'templateThemeId',
			A.[TemplateThemeItemId]								'templateThemeItemId',
			ISNULL(D.[Query], A.[TemplateThemeItemId])			'query', -- Si es NULL se mostrara valor de TemplateThemeItem campo ID.
			ISNULL(C.[Channel], AI.[Channel])					'interactionType',
			-- REPLACE(A.[TranscriptionId], '_0_1', '')			'transcriptionId',

			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN REPLACE(A.[TranscriptionId], '_0_1', '') 
				ELSE AI.[ID] END								'transcriptionId',

			C.[Section]											'section',
			C.[Part]											'part',
			AI.[Folder]											'folder',
			AI.[Campaign]										'campaign',
			--ISNULL(C.[Agent], '')								'agent'
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[Agent], '') 
				ELSE ISNULL(AI.[FirstTakedAgent], '') END		'agent',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsTranscribed], '') 
				ELSE ISNULL(AI.[IsTranscribed], '') END			'isTranscribed',
			CASE WHEN C.[Channel] IS NOT NULL
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsCancel], '') 
				ELSE ISNULL(AI.[IsCancel], '') END				'isCancel'
		FROM 
			[dbo].[StudyResultSummaryDetail] A (NOLOCK)
			INNER JOIN [dbo].[Study] S (NOLOCK) 
				ON S.[VCC] = A.[VCC] 
					AND S.[Name] = A.[StudyId]
			INNER JOIN [dbo].[TemplateThemeItem] B (NOLOCK) 
				ON S.[TemplateId] = B.[TemplateId] 
					AND A.[TemplateThemeId] = B.[TemplateThemeId] 
					AND A.[TemplateThemeItemId] = B.[ID]
			
			LEFT JOIN [dbo].[Audio] C (NOLOCK)  -- (!)SE CAMBIO EL INNER POR LEFT PARA CONSIDER LAS TRANSCRIPCIONES DE CANALES ASINCRINOS
				ON A.[TranscriptionId] = CONCAT(C.[ID], '_', C.[Section], '_', C.[Part])
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON S.[VCC] = PA.[VCC] 
					AND S.[ProcessId] = PA.[ProcessId] 
					AND A.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])

			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON A.[TranscriptionId] = CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_'))
			LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON PAI.[VCC] = S.[VCC]
					AND PAI.[ProcessId] = S.[ProcessId]
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = A.[TranscriptionId]

			LEFT JOIN [dbo].[GenericQuery] D (NOLOCK) 
				ON B.[VCC] = D.[VCC] 
					AND  B.[ID] = D.[Name] -- Se agrega LEFT JOIN para poder mostrar.
		WHERE
			B.[VCC] IN ('system', @VCC) AND
			A.[StudyId] = @StudyId
	END
	ELSE
	BEGIN
		-- ESTE BLOQUE NUNCA SE USA, EL UNICO LLAMADO DE LA FUNCION INSERTA DIRECTAMENTE NULL PARA LA VARIABLE @NewStructure
		SELECT
			A.[TemplateThemeId]									'templateThemeId',
			A.[TemplateThemeItemId]								'templateThemeItemId',
			D.[Query]											'query',
			ISNULL(C.[Channel], AI.[Channel])					'interactionType',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN A.[TranscriptionId] 
				ELSE AI.[ID] END								'transcriptionId',
			C.[Section]											'section',
			C.[Part]											'part',
			AI.[Folder]											'folder',
			AI.[Campaign]										'campaign',
			--ISNULL(C.[Agent], '')								'agent',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[Agent], '') 
				ELSE ISNULL(AI.[FirstTakedAgent], '') END		'agent',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsTranscribed], '') 
				ELSE ISNULL(AI.[IsTranscribed], '') END			'isTranscribed',
			CASE WHEN C.[Channel] IS NOT NULL
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsCancel], '') 
				ELSE ISNULL(AI.[IsCancel], '') END				'isCancel'
			
		FROM 
			[dbo].[StudyResultSummaryDetail] A (NOLOCK)
			INNER JOIN [dbo].[Study] S (NOLOCK) 
				ON S.[VCC] = A.[VCC] 
					AND S.[Name] = A.[StudyId]
			INNER JOIN [dbo].[TemplateThemeItem] B (NOLOCK) 
				ON S.[TemplateId] = B.[TemplateId] 
					AND A.[TemplateThemeId] = B.[TemplateThemeId] 
					AND A.[TemplateThemeItemId] = B.[ID]
			
			LEFT JOIN [dbo].[Audio] C (NOLOCK) 
				ON A.[TranscriptionId] = CONCAT(C.[ID], '_', C.[Section], '_', C.[Part])
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON S.[VCC] = PA.[VCC] 
					AND S.[ProcessId] = PA.[ProcessId] 
					AND A.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON A.[TranscriptionId] = CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_'))
			LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON PAI.[VCC] = S.[VCC]
					AND PAI.[ProcessId] = S.[ProcessId]
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = A.[TranscriptionId]

			INNER JOIN [dbo].[GenericQuery] D (NOLOCK) 
				ON B.[VCC] = D.[VCC] 
					AND  B.[ID] = D.[Name]
		WHERE
			B.[VCC] IN ('system', @VCC) AND
			A.[StudyId] = @StudyId
	END
	
	SET NOCOUNT OFF
END
GO
PRINT N'Refreshing [dbo].[BulkAudio]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkAudio]';


GO
PRINT N'Refreshing [dbo].[BulkInteractionQueryStudy]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkInteractionQueryStudy]';


GO
PRINT N'Refreshing [dbo].[BulkProcessInteraction]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkProcessInteraction]';


GO
PRINT N'Refreshing [dbo].[BulkProgress]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[BulkProgress]';


GO
PRINT N'Refreshing [dbo].[EvaluateProcessStatus]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[EvaluateProcessStatus]';


GO
PRINT N'Refreshing [dbo].[GetAllegroBillingInteractionsForEvaluationPlan]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAllegroBillingInteractionsForEvaluationPlan]';


GO
PRINT N'Refreshing [dbo].[GetAudio]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAudio]';


GO
PRINT N'Refreshing [dbo].[GetAudios]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAudios]';


GO
PRINT N'Refreshing [dbo].[GetAudiosByProcessId]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAudiosByProcessId]';


GO
PRINT N'Refreshing [dbo].[GetAudiosProcess]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAudiosProcess]';


GO
PRINT N'Refreshing [dbo].[GetEvaluationPlanCognitiveInteractionTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationPlanCognitiveInteractionTask]';


GO
PRINT N'Refreshing [dbo].[GetEvaluationPlanCognitiveTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationPlanCognitiveTask]';


GO
PRINT N'Refreshing [dbo].[GetExecuteTemplateProcess]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetExecuteTemplateProcess]';


GO
PRINT N'Refreshing [dbo].[GetInteractionsCognitive]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionsCognitive]';


GO
PRINT N'Refreshing [dbo].[GetInteractionsResult]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionsResult]';


GO
PRINT N'Refreshing [dbo].[GetInteractionTranscription]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionTranscription]';


GO
PRINT N'Refreshing [dbo].[GetLinguisticProcessStatus]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetLinguisticProcessStatus]';


GO
PRINT N'Refreshing [dbo].[GetPendingAudios]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetPendingAudios]';


GO
PRINT N'Refreshing [dbo].[GetProcessInteractionsHistory]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetProcessInteractionsHistory]';


GO
PRINT N'Refreshing [dbo].[GetProcessInteractionsTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetProcessInteractionsTask]';


GO
PRINT N'Refreshing [dbo].[GetProcessSummary]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetProcessSummary]';


GO
PRINT N'Refreshing [dbo].[GetStudiesInProcess]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetStudiesInProcess]';


GO
PRINT N'Refreshing [dbo].[GetStudyCognitiveInteractionsTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetStudyCognitiveInteractionsTask]';


GO
PRINT N'Refreshing [dbo].[SaveAudio]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveAudio]';


GO
PRINT N'Refreshing [dbo].[SaveAudioLinguisticProcess]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveAudioLinguisticProcess]';


GO
PRINT N'Update complete.';


GO
