-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.5.2'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.6.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- ACTUALIZAMOS EL TIPO DE AMBIENTE A INCONCERT PARA LOS FORMATOS EXISTENTES
    UPDATE [dbo].[EvaluationFormat] SET [EnvironmentType] = 'INCONCERT'

    -- AGREGAMOS LOS PERMISOS DE ENVIROMENT PARA SYSTEM
    -- INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (N'system', N'quality_admin', N'PERM_GROUP_ENVIRONMENT_ADD', GETUTCDATE())
    -- INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (N'system', N'quality_admin', N'PERM_GROUP_ENVIRONMENT_DELETE', GETUTCDATE())
    -- INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (N'system', N'quality_admin', N'PERM_GROUP_ENVIRONMENT_EDIT', GETUTCDATE())
    -- INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (N'system', N'quality_admin', N'PERM_GROUP_ENVIRONMENT_VIEW', GETUTCDATE())

    -- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS VCC QUE ACTUALIZAREMOS
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        -- SELECCIONAMOS LOS VCC EXISTENTES
        SELECT [VCC]
			FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        -- AGREGAMOS LOS PERMISOS DEL MODULO ENVIRONMENT AL VCC ACTUAL DEL RECORRIDO
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_ENVIRONMENT_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_ENVIRONMENT_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_ENVIRONMENT_EDIT', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_ENVIRONMENT_VIEW', GETUTCDATE())

        -- ACTUALIZAMOS EL TIPO DE PERMISO PARA ENVIRONMENT A SHARED
        UPDATE [dbo].[RolePermissionsGroup] SET [Type] = 'SHARED' WHERE [Category] = 'CATEG_GROUP_ENVIRONMENT' AND [VCC] = @CURRENT_VCC_TO_UPDATE
        
        -- AGREGAMOS EL PERMISO PARA DESCARGAR LA PLANTILLA CSV DESDE EVALUATION PLAN CREATE
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/process/download_template_audio_csv/', N'api', GETUTCDATE())

        -- AGREGAMOS EL PERMISO PARA SUBIR Y DESCARGAR EL ARCHIVO VSC
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/process/upload_file/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/process/download_file/', N'api', GETUTCDATE())

        -- AGREGAMOS EL PERMISO PARA SOLICITAR LOS VALORES PARA FILTROS EXTRAIDOS DEL CONTENIDO CSV
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_ADD', N'api/process/get_csv_filters', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/process/get_csv_filters', N'api', GETUTCDATE())

        -- ACTUALIZAMOS EL ENVIRONMENTID DE LOS PLANES EXISTENTES A ALLEGRO
        UPDATE [dbo].[EvaluationPlan] SET [EnvironmentId] = 'Ambiente' WHERE [EnvironmentId] IS NULL AND [Instance] = @CURRENT_VCC_TO_UPDATE

        -- ELIMINAMOS UNA RUTA QUE YA NO SE USA
        DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @CURRENT_VCC_TO_UPDATE AND [Route] = 'api/process/get_csv_data'

        -- PERMISO PARA ELIMINAR DE REDIS EL CONTENIDO DEL CSV SUBIDO
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/process/delete_csv_temporal/', N'api', GETUTCDATE())

        -- SECOND UPDATE 20200908
        -- ELIMINAMOS LAS RUTAS QUE YA NO SE USAN
        DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @CURRENT_VCC_TO_UPDATE AND [GroupId] = 'PERM_GROUP_AUDITOR_INBOX' AND [Route] = 'api/dispositioncode/get'
        DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @CURRENT_VCC_TO_UPDATE AND [GroupId] = 'PERM_GROUP_ADMIN_INBOX' AND [Route] = 'api/dispositioncode/get'
        DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @CURRENT_VCC_TO_UPDATE AND [GroupId] = 'PERM_GROUP_AGENT_INBOX' AND [Route] = 'api/dispositioncode/get'

        -- AGREGAMOS LOS PERMISOS PARA LLENAR LOS FILTROS CON DATA DE LAS EVALUACIONES
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_QUALITY_DASHBOARD', N'api/campaign/evaluation', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_QUALITY_DASHBOARD', N'api/agent/evaluation', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_AUDITOR_INBOX', N'api/dispositioncode/evaluation', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_ADMIN_INBOX', N'api/dispositioncode/evaluation', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_AGENT_INBOX', N'api/dispositioncode/evaluation', N'api', GETUTCDATE())
        

        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH