-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.6.3'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.7.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION

    
    -- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS VCC QUE ACTUALIZAREMOS
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        -- SELECCIONAMOS LOS VCC EXISTENTES
        SELECT [VCC]
			FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        -- AGREGAMOS LOS PERMISOS DEL MODULO ENVIRONMENT AL VCC ACTUAL DEL RECORRIDO
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_NLU_ADD', N'api/nlu/upload_file/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_NLU_ADD', N'api/nlu/delete_file_temporal', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_NLU_ADD', N'api/nlu/export_nlu', N'api', GETUTCDATE())

        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR


    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH



