PRINT N'Quitando [dbo].[DF_VCC_Type]...';


GO
ALTER TABLE [dbo].[VCC] DROP CONSTRAINT [DF_VCC_Type];


GO
PRINT N'Iniciando recompilación de la tabla [dbo].[VCC]...';


GO
BEGIN TRANSACTION;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

SET XACT_ABORT ON;

CREATE TABLE [dbo].[tmp_ms_xx_VCC] (
    [VCC]                  VARCHAR (50) NOT NULL,
    [ID]                   VARCHAR (50) NOT NULL,
    [Type]                 VARCHAR (50) CONSTRAINT [DF_VCC_Type] DEFAULT ('ALLEGRO') NOT NULL,
    [Description]          VARCHAR (50) NULL,
    [CognitiveEngine]      VARCHAR (25) NULL,
    [Country]              VARCHAR (50) NULL,
    [Language]             VARCHAR (50) NULL,
    [TimeZone]             VARCHAR (50) NULL,
    [CreatedDate]          DATETIME     NULL,
    [CreatedByUserId]      VARCHAR (50) NULL,
    [LastModifiedDate]     DATETIME     NULL,
    [LastModifiedByUserId] VARCHAR (50) NULL,
    [TimeStamp]            DATETIME     NULL,
    CONSTRAINT [tmp_ms_xx_constraint_PK_VCC1] PRIMARY KEY CLUSTERED ([VCC] ASC, [ID] ASC)
);

IF EXISTS (SELECT TOP 1 1 
           FROM   [dbo].[VCC])
    BEGIN
        INSERT INTO [dbo].[tmp_ms_xx_VCC] ([VCC], [ID], [Type], [Description], [Country], [Language], [TimeZone], [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], [TimeStamp])
        SELECT   [VCC],
                 [ID],
                 [Type],
                 [Description],
                 [Country],
                 [Language],
                 [TimeZone],
                 [CreatedDate],
                 [CreatedByUserId],
                 [LastModifiedDate],
                 [LastModifiedByUserId],
                 [TimeStamp]
        FROM     [dbo].[VCC]
        ORDER BY [VCC] ASC, [ID] ASC;
    END

DROP TABLE [dbo].[VCC];

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_VCC]', N'VCC';

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_constraint_PK_VCC1]', N'PK_VCC', N'OBJECT';

COMMIT TRANSACTION;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED;


GO
PRINT N'Modificando [dbo].[GetVCC]...';


GO
ALTER PROCEDURE [dbo].[GetVCC](
	@ID varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[ID]							'id',
		[TYPE]							'type',
		[Description]					'description',
		[CognitiveEngine]				'cognitiveEngine',
		[Country]						'country',
		[Language]						'language',
		[TimeZone]						'timeZone',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [VCC] (NOLOCK)
	WHERE [ID] = @ID

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetVCCs]...';


GO
ALTER PROCEDURE [dbo].[GetVCCs] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[Type]							'type',
		[CognitiveEngine]				'cognitiveEngine',
		[Country]						'country',
		[Language]						'language',
		[TimeZone]						'timeZone',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [VCC] (NOLOCK)
	WHERE [VCC] <> 'system'
		AND [VCC] <> 'default'
		AND ISNULL([ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Country], '') LIKE CASE WHEN @SearchCriteria = 'country' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveVcc]...';


GO
ALTER PROCEDURE [dbo].[SaveVcc] (
	@VCC Varchar(50) = NULL, 
	@ID Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@Type VARCHAR(50) = NULL,
	@CognitiveEngine VARCHAR(25) = NULL,
	@Country Varchar(50) = NULL,
	@Language Varchar(50) = NULL,
	@TimeZone Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [VCC]
		SET [Description] = @Description,
			[Type] = @Type,
			[Country] = @Country,
			[CognitiveEngine] = @CognitiveEngine,
			[Language] = @Language,
			[TimeZone] = @TimeZone,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [VCC] (
			[VCC], [ID], [Description], [Type], [Country], 
			[CognitiveEngine], [Language], [TimeZone], [CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @ID, @Description, @Type, @Country, 
			@CognitiveEngine, @Language, @TimeZone, @CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[DeleteIntent]...';


GO

ALTER PROCEDURE [dbo].[DeleteIntent] (
	@VCC Varchar(50) = NULL, 
	@AppId VARCHAR(100) = NULL,
	@IntentId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- ELIMINAMOS EL INTENT DE LA TABLA INTENT
	DELETE FROM [Intent]
		WHERE [VCC] = @VCC
			AND [AppId] = @AppId
 			AND [IntentId] = @IntentId

	-- ELIMINAMOS LOS UTTERANCES REGISTRADOS CON EL APPID E INTENTID
	DELETE FROM [Utterance]
		WHERE [VCC] = @VCC 
			AND [AppId] = @AppId
			AND [IntentId] = @IntentId

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[DeleteNLU]...';


GO

ALTER PROCEDURE [dbo].[DeleteNLU] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ID DEL NLU
	DECLARE @APP_ID VARCHAR(100) = NULL
	
	-- ASIGNAMOS EL APPID A LA VARIABLE
	SELECT @APP_ID = N.[AppId]
		FROM [dbo].[NLU] N (NOLOCK)
		WHERE N.[VCC] = @VCC 
			AND N.[Name] = @Name

	-- ELIMINAMOS EL MODELO DE LA TABLA NLU
	DELETE FROM [dbo].[NLU]
		WHERE [VCC] = @VCC
			AND [AppId] = @APP_ID

	-- ELIMINAMOS LOS INTENTS DE LA TABLA INTENT
	DELETE FROM [dbo].[Intent]
		WHERE [VCC] = @VCC
			AND [AppId] = @APP_ID

	-- ELIMINAMOS LOS UTTERANCES LA TABLA UTTERANCE
	DELETE FROM [dbo].[Utterance]
		WHERE [VCC] = @VCC
			AND [AppID] = @APP_ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetEvaluationPlanCognitiveInteractionTask]...';


GO

ALTER PROCEDURE [dbo].[GetEvaluationPlanCognitiveInteractionTask] (
    @VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	SELECT DISTINCT
			A.Instance					'vcc'
			,A.ProcessId				'processId'
			,C.Campaign					'campaign'
			,B.[InteractionType]		'interactionType'
			,A.[Id]						'evaluationPlanId'
			,B.IdInteraction			'id'
			,C.Section					'section'
			,C.Part						'part'
			,NULL						'folder'
			,F.nluId					'nluId'
			,F.appId					'appId'
			,C.[FileName]				'fileName'
			,F.[cognitiveId]
			,V.[CognitiveEngine]		'cognitiveEngine'
	FROM [EvaluationPlan] A (NOLOCK)
	INNER JOIN [InteractionEvaluationPlan] B (NOLOCK) 
		ON A.Instance = B.Instance 
			AND A.Id = B.IdEvaluationPlan
	INNER JOIN [dbo].[Audio] C WITH (NOLOCK) 
		ON A.Instance = C.VCC 
			AND B.IdInteraction = C.ID 
	INNER JOIN [AbilityEvaluationFormat] D 
		ON A.Instance = D.Instance 
			AND A.IdEvaluationFormat = D.IdEvaluationFormat
	INNER JOIN [Ability] E 
		ON A.Instance = E.Instance 
			AND D.IdAbility = E.ID 
			AND E.QueryName IS NOT NULL
	INNER JOIN (SELECT DISTINCT
				A.VCC 'vcc'
				,A.[Name] + '|'+ B.[Name] 'cognitiveId'
				,A.[Name] 'nluId'
				,A.AppId 'appId'
				,B.[IntentId] 'intentId',B.Name
			FROM NLU A (NOLOCK)
			INNER JOIN Intent B (NOLOCK) 
				ON A.VCC = B.VCC
					AND B.AppId = A.AppId) F 
		ON A.Instance = F.VCC 
			--AND E.ID = F.cognitiveId
			AND E.Query = F.cognitiveId
			AND E.QueryName = F.intentId
	LEFT JOIN [dbo].[VCC] V (NOLOCK)
		ON V.[ID] = A.[Instance]
	WHERE A.[Instance] = @VCC
		AND A.[ID] = @ID
	
	SELECT DISTINCT
			A.Instance					'vcc'
			,A.ProcessId				'processId'
			,C.[Campaign]				'campaign'
			,B.[InteractionType]		'interactionType'
			,A.[Id]						'evaluationPlanId'
			,B.IdInteraction			'id'
			,NULL						'section'
			,NULL						'part'
			,C.[EndDate]				'endDate'
			,C.[Folder]					'folder'
			,C.[DispositionCode]		'dispositionCode'
			,F.nluId					'nluId'
			,F.appId					'appId'
			,C.[FileName]				'fileName'
			,F.[cognitiveId]
			,V.[CognitiveEngine]		'cognitiveEngine'
	FROM [EvaluationPlan] A (NOLOCK)
	INNER JOIN [InteractionEvaluationPlan] B (NOLOCK) 
		ON A.Instance = B.Instance 
			AND A.Id = B.IdEvaluationPlan
	INNER JOIN [dbo].[AsyncInteraction] C WITH (NOLOCK) 
		ON A.Instance = C.VCC 
			AND B.IdInteraction = C.ID
			AND B.FormattedEndDate = C.Folder
	INNER JOIN [AbilityEvaluationFormat] D 
		ON A.Instance = D.Instance 
			AND A.IdEvaluationFormat = D.IdEvaluationFormat
	INNER JOIN [Ability] E 
		ON A.Instance = E.Instance 
			AND D.IdAbility = E.ID 
			AND E.QueryName IS NOT NULL
	INNER JOIN (SELECT DISTINCT
				A.VCC 'vcc'
				,A.[Name] + '|'+ B.[Name] 'cognitiveId'
				,A.[Name] 'nluId'
				,A.AppId 'appId'
				,B.[IntentId] 'intentId',B.Name
			FROM NLU A (NOLOCK)
			INNER JOIN Intent B (NOLOCK) 
				ON A.VCC = B.VCC
					AND B.AppId = A.AppId) F 
		ON A.Instance = F.VCC 
			--AND E.ID = F.cognitiveId
			AND E.Query = F.cognitiveId
			AND E.QueryName = F.intentId
	LEFT JOIN [dbo].[VCC] V (NOLOCK)
		ON V.[ID] = A.[Instance]
	WHERE A.[Instance] = @VCC
		AND A.[ID] = @ID

SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetIntents]...';


GO
ALTER PROCEDURE [dbo].[GetIntents] (
	@VCC Varchar(50) = NULL, 
	@AppId VARCHAR(100) = NULL,
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL,
	@Intents NVARCHAR(MAX)
	)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON
	SELECT
			[Name]						    ''name'',
			[IntentId]						''intentId'',
			[Color]							''color'',
			[CreatedDate]					''createdDate'',
			[CreatedByUserId]				''createdByUserId'',
			[LastModifiedDate]				''lastModifiedDate'',
			[LastModifiedByUserId]			''lastModifiedByUserId''
		FROM [Intent] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + '''
			AND [AppId] = ''' + @AppId + '''
			AND [IntentId] IN (''' + @Intents + ''')
			AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
		ORDER BY [LastModifiedDate] ASC, [Name] DESC
	'
	SET @SQL += ' SET NOCOUNT OFF'
	EXEC(@SQL)
END
GO
PRINT N'Actualizando [dbo].[DeleteVCC]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[DeleteVCC]';


GO
PRINT N'Actualización completada.';


GO
