-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '0.9.0'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '0.9.1'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION


    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS VCC QUE ACTUALIZAREMOS
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        -- SELECCIONAMOS LOS VCC EXISTENTES
        SELECT [VCC]
			FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR
    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 

        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        -- AGREGAMOS LA NUEVA CATEGORIA DE PERMISOS
        INSERT [dbo].[CategoryPermissions] ([VCC], [Category], [Position], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_DIARIZATION_PHRASE', 19, GETUTCDATE())
       
        -- AGREGAMOS LOS GRUPOS DE PERMISOS CORRESPONDIENTES
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_DIARIZATION_PHRASE_ADD', N'CATEG_GROUP_DIARIZATION_PHRASE', GETUTCDATE(), N'SHARED')
        -- INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_DIARIZATION_PHRASE_EDIT', N'CATEG_GROUP_DIARIZATION_PHRASE', GETUTCDATE(), N'SHARED')
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_DIARIZATION_PHRASE_DELETE', N'CATEG_GROUP_DIARIZATION_PHRASE', GETUTCDATE(), N'SHARED')
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [GroupId], [Category], [TimeStamp], [Type]) VALUES (@CURRENT_VCC_TO_UPDATE, N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'CATEG_GROUP_DIARIZATION_PHRASE', GETUTCDATE(), N'SHARED')
        
        -- AGREGAMOS LAS RELACIONES ROL-PERMISO CORRESPONDIENTES
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_DIARIZATION_PHRASE_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_DIARIZATION_PHRASE_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_DIARIZATION_PHRASE_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_DIARIZATION_PHRASE_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'quality_admin', N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', GETUTCDATE())
        
        -- AGREGAMOS LOS PERMISOS PARA EL NUEVO MODULO
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'speechanalytic/provisioning/diarization-phrase', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'api/nlu/get_nlus/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'speechanalytic/provisioning/diarization-phrase/view/', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'api/nlu/get_nlu/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_VIEW', N'api/utterance/get_utterances/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_ADD', N'speechanalytic/provisioning/diarization-phrase/add/', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_ADD', N'api/utterance/create_utterance', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_DIARIZATION_PHRASE_DELETE', N'api/utterance/delete_utterance', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_ADD', N'api/diarization/get_model_availability', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_EDIT', N'api/diarization/get_model_availability', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_ADD', N'api/diarization/get_model_availability', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_EVALUATIONPLAN_EDIT', N'api/diarization/get_model_availability', N'api', GETUTCDATE())

        -- MODELOS POR DEFECTO PARA DIARIZACION
        INSERT [dbo].[NLU] ([VCC], [Name], [Culture], [Description], [AppId], [IsSystemApp], [CreatedDate], [CreatedByUserId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'diarization-es', N'es', N'DIARIZATION_LANGUAGE_ES', @CURRENT_VCC_TO_UPDATE + N'::diarization-es', 1, GETUTCDATE(), N'system', GETUTCDATE())
        INSERT [dbo].[NLU] ([VCC], [Name], [Culture], [Description], [AppId], [IsSystemApp], [CreatedDate], [CreatedByUserId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'diarization-en', N'en', N'DIARIZATION_LANGUAGE_EN', @CURRENT_VCC_TO_UPDATE + N'::diarization-en', 1, GETUTCDATE(), N'system', GETUTCDATE())
        INSERT [dbo].[NLU] ([VCC], [Name], [Culture], [Description], [AppId], [IsSystemApp], [CreatedDate], [CreatedByUserId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'diarization-pt', N'pt', N'DIARIZATION_LANGUAGE_PT', @CURRENT_VCC_TO_UPDATE + N'::diarization-pt', 1, GETUTCDATE(), N'system', GETUTCDATE())
        INSERT [dbo].[NLU] ([VCC], [Name], [Culture], [Description], [AppId], [IsSystemApp], [CreatedDate], [CreatedByUserId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'diarization-fr', N'fr', N'DIARIZATION_LANGUAGE_FR', @CURRENT_VCC_TO_UPDATE + N'::diarization-fr', 1, GETUTCDATE(), N'system', GETUTCDATE())

        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    
    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH