PRINT N'Quitando [dbo].[DF_NLU_IsSystemApp]...';


GO
ALTER TABLE [dbo].[NLU] DROP CONSTRAINT [DF_NLU_IsSystemApp];


GO
PRINT N'Iniciando recompilación de la tabla [dbo].[NLU]...';


GO
BEGIN TRANSACTION;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

SET XACT_ABORT ON;

CREATE TABLE [dbo].[tmp_ms_xx_NLU] (
    [VCC]                  VARCHAR (50)  NOT NULL,
    [Name]                 VARCHAR (50)  NOT NULL,
    [Culture]              VARCHAR (50)  NULL,
    [Description]          VARCHAR (50)  NULL,
    [AppId]                VARCHAR (100) NULL,
    [SubscriptionKey]      VARCHAR (50)  NULL,
    [IsPublished]          BIT           NULL,
    [LastPublishedDate]    DATETIME      NULL,
    [IsTrained]            BIT           NULL,
    [NeedTrain]            BIT           CONSTRAINT [DF_NLU_NeedTrain] DEFAULT ((0)) NULL,
    [LastTrainedDate]      DATETIME      NULL,
    [IsSystemApp]          BIT           CONSTRAINT [DF_NLU_IsSystemApp] DEFAULT ((0)) NULL,
    [CreatedDate]          DATETIME      NULL,
    [CreatedByUserId]      VARCHAR (50)  NULL,
    [LastModifiedDate]     DATETIME      NULL,
    [LastModifiedByUserId] VARCHAR (50)  NULL,
    [TimeStamp]            DATETIME      NULL,
    CONSTRAINT [tmp_ms_xx_constraint_PK_NLU1] PRIMARY KEY CLUSTERED ([VCC] ASC, [Name] ASC)
);

IF EXISTS (SELECT TOP 1 1 
           FROM   [dbo].[NLU])
    BEGIN
        INSERT INTO [dbo].[tmp_ms_xx_NLU] ([VCC], [Name], [Culture], [Description], [AppId], [SubscriptionKey], [IsPublished], [LastPublishedDate], [IsTrained], [LastTrainedDate], [IsSystemApp], [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], [TimeStamp])
        SELECT   [VCC],
                 [Name],
                 [Culture],
                 [Description],
                 [AppId],
                 [SubscriptionKey],
                 [IsPublished],
                 [LastPublishedDate],
                 [IsTrained],
                 [LastTrainedDate],
                 [IsSystemApp],
                 [CreatedDate],
                 [CreatedByUserId],
                 [LastModifiedDate],
                 [LastModifiedByUserId],
                 [TimeStamp]
        FROM     [dbo].[NLU]
        ORDER BY [VCC] ASC, [Name] ASC;
    END

DROP TABLE [dbo].[NLU];

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_NLU]', N'NLU';

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_constraint_PK_NLU1]', N'PK_NLU', N'OBJECT';

COMMIT TRANSACTION;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED;


GO
PRINT N'Modificando [dbo].[GetNLU]...';


GO
--exec [GetNLU] NULL,'test2'
ALTER PROCEDURE [dbo].[GetNLU] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[Name]						    'name',
		[Culture]						'culture',
		[Description]					'description',
		[AppId]							'appId',
		[SubscriptionKey]				'subscriptionKey',
		[IsSystemApp]					'isSystemApp',
		[NeedTrain]						'needTrain',
		[LastTrainedDate]				'lastTrainedDateTime',
		[LastPublishedDate]				'lastPublishedDateTime',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [NLU] (NOLOCK)
	WHERE [VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN [VCC] ELSE @VCC END 
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetNLUByAppId]...';


GO
--exec [GetNLU] NULL,'test2'
ALTER PROCEDURE [dbo].[GetNLUByAppId] (
	@VCC Varchar(50) = NULL, 
	@AppId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT 
			N.[VCC]							'vcc',
			N.[Name]						'name',
			N.[Culture]						'culture',
			N.[Description]					'description',
			N.[AppId]						'appId',
			N.[SubscriptionKey]				'subscriptionKey',
			N.[IsSystemApp]					'isSystemApp',
			N.[NeedTrain]					'needTrain',
			N.[LastTrainedDate]				'lastTrainedDateTime',
			N.[LastPublishedDate]				'lastPublishedDateTime',
			N.[CreatedDate]					'createdDate',
			N.[CreatedByUserId]				'createdByUserId',
			N.[LastModifiedDate]			'lastModifiedDate',
			N.[LastModifiedByUserId]		'lastModifiedByUserId'
	FROM [dbo].[NLU] N (NOLOCK)
	WHERE N.[VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN N.[VCC] ELSE @VCC END 
		AND N.[AppId] = @AppId
	
	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveNLU]...';


GO

ALTER PROCEDURE [dbo].[SaveNLU] (
	@VCC Varchar(50) = NULL, 					
	@Name Varchar(50) = NULL,					
	@Culture Varchar(50) = NULL,
	@Description Varchar(50) = NULL,			
	@AppId Varchar(100) = NULL,					
	@SubscriptionKey Varchar(50) = NULL,
	@IsSystemApp BIT = 0,
	@IsPublished BIT = NULL,
	@LastPublishedDate DATETIME = NULL,
	@IsTrained BIT = NULL,
	@NeedTrain BIT = NULL,
	@LastTrainedDate DATETIME = NULL,
	@CreatedDate Datetime = NULL,               
	@CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	-- SI LA VARIABLE LLEGA NULL, LA SETTEAMOS COMO 0
	IF @IsSystemApp IS NULL 
		SET @IsSystemApp = 0

	UPDATE [NLU]
	SET [Culture] = @Culture,
		[Description] = @Description,
		[AppId] = @AppId,
		[IsPublished] = CASE WHEN @IsPublished IS NULL THEN [IsPublished] ELSE @IsPublished END,
		[LastPublishedDate] = CASE WHEN @LastPublishedDate IS NULL THEN [LastPublishedDate] ELSE @LastPublishedDate END,
		[IsSystemApp] = CASE WHEN @IsSystemApp IS NULL THEN [IsSystemApp] ELSE @IsSystemApp END,
		[SubscriptionKey] = CASE WHEN @SubscriptionKey IS NULL THEN [SubscriptionKey] ELSE @SubscriptionKey END,
		[IsTrained] = CASE WHEN @IsTrained IS NULL THEN [IsTrained] ELSE @IsTrained END,
		[NeedTrain] = CASE WHEN @NeedTrain IS NULL THEN [NeedTrain] ELSE @NeedTrain END,
		[LastTrainedDate] = CASE WHEN @LastTrainedDate IS NULL THEN [LastTrainedDate] ELSE @LastTrainedDate END,
		-- [CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [NLU] (
			[VCC], [Name], [Culture],
			[Description], [AppId], [SubscriptionKey],
			[IsSystemApp], [CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Culture,
			@Description, @AppId, @SubscriptionKey,
			@IsSystemApp, GETUTCDATE(), @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetNLUs]...';


GO
ALTER PROCEDURE [dbo].[GetNLUs] (
	@VCC Varchar(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL,
	@Apps NVARCHAR(MAX),
	@IsSystemApp BIT = NULL
	)
AS
BEGIN
	-- EN CASO EL FLAG LLEGUE NULL LO SETTEAMOS COMO 0
	
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON
	SELECT
		[VCC]							''vcc'',
		[Name]						    ''name'',
		[Culture]						''culture'',
		[Description]					''description'',
		[AppId]							''appId'',
		[SubscriptionKey]				''subscriptionKey'',
		[IsSystemApp]					''isSystemApp'',
		[NeedTrain]						''needTrain'',
		[IsTrained]						''isTrained'',
		[LastTrainedDate]				''lastTrainedDateTime'',
		[IsPublished]					''isPublished'',
		[LastPublishedDate]				''lastPublishedDateTime'',
		[CreatedDate]					''createdDate'',
		[CreatedByUserId]				''createdByUserId'',
		[LastModifiedDate]				''lastModifiedDate'',
		[LastModifiedByUserId]			''lastModifiedByUserId''
	FROM [NLU] (NOLOCK)
	WHERE ' 
	
	-- EN CASO BUSQUEMOS MODELOS DE TODOS LOS VCC
	IF (@VCC = 'ALL') 
		SET @SQL += '
			[VCC] IS NOT NULL '
	ELSE 
		SET @SQL += '
		[VCC] = ''' + @VCC + ''' '

	-- EN CASO BUSQUEMOS MODELOS DEL SISTEMA NO FILTRAMOS POR NOMBRES
	IF (@IsSystemApp = 0)
		SET @SQL += '
		AND [AppId] IN (''' + @Apps + ''') '

	SET @SQL += '
		AND [IsSystemApp] = ' + CASE WHEN @IsSystemApp = 1 THEN '1' ELSE '0' END + ' '

	IF @SearchCriteria IS NOT NULL AND @SearchCriteria  IS NOT NULL
		SET @SQL += '
		AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
		AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'

	SET @SQL += '
	ORDER BY [LastModifiedDate] ASC, [Name] DESC'

	SET @SQL += ' SET NOCOUNT OFF'

	EXEC(@SQL)
END
GO
PRINT N'Actualizando [dbo].[DeleteNLU]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[DeleteNLU]';


GO
PRINT N'Actualizando [dbo].[GetAbilitys]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAbilitys]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationFormats]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationFormats]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationPlanCognitiveInteractionTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationPlanCognitiveInteractionTask]';


GO
PRINT N'Actualizando [dbo].[GetEvaluationPlanCognitiveTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetEvaluationPlanCognitiveTask]';


GO
PRINT N'Actualizando [dbo].[GetInteractionsCognitive]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInteractionsCognitive]';


GO
PRINT N'Actualizando [dbo].[GetStudiesInProcess]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetStudiesInProcess]';


GO
PRINT N'Actualizando [dbo].[GetStudyCognitiveInteractionsTask]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetStudyCognitiveInteractionsTask]';


GO
PRINT N'Actualizando [dbo].[GetTemplates]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetTemplates]';


GO
PRINT N'Actualizando [dbo].[GetUniqueNLUByIntents]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetUniqueNLUByIntents]';


GO
PRINT N'Actualización completada.';


GO
